/********************************************************************
 * DESCRIPTION : initialize Interrupt Descriptor Table
 * FILE NAME : InitIDT.c
 *******************************************************************/

/* header file for initializing IDTs */
#include <IDT.h>
/* header file for initializing IRQs */
#include <IRQ.h>


/******************************************************************
 * NAME : SetIntrGate
 * SYNOPSIS : initialize interrupt gate
 * EXTERNAL EFFECTS : IDT[]
 * ARGUMENTS : A_iIntNum interrupt number
 *             A_pHandler pointer of handler function
 * RETURNS : N/A
 * ERRNOS : N/A
 ******************************************************************/
void SetIntrGate(int A_nIntNum, void *A_pHandler)
{
		if (A_nIntNum < IDT_SIZE) {
				IDT[A_nIntNum].OffsetHigh = (unsigned short)( ((unsigned long)A_pHandler >> 16));
				IDT[A_nIntNum].Type = DESTYPE_INT;		/* interrupt descriptor */
				IDT[A_nIntNum].SegSelector = KERNEL_CODE;	/* kernel code segment */
				IDT[A_nIntNum].OffsetLow = (unsigned short)((unsigned long)A_pHandler & 0xffff);
		} 
}

/******************************************************************
 * NAME : SetSystemrGate
 * SYNOPSIS : initialize system gate
 * EXTERNAL EFFECTS : IDT[]
 * ARGUMENTS : A_nIntNum exception number
 *             A_pHandler address of exception handler
 * RETURNS : N/A
 * ERRNOS : N/A
 ******************************************************************/
void SetSystemGate(int A_nIntNum, void *A_pHandler)
{
		if (A_nIntNum < IDT_SIZE) {
				IDT[A_nIntNum].OffsetHigh = (unsigned short)((unsigned long)A_pHandler & 0xffff0000) >> 16;
				IDT[A_nIntNum].Type = DESTYPE_TRAP;			/* descriptor type */
				IDT[A_nIntNum].SegSelector = KERNEL_CODE;	/* kernel code segment */
				IDT[A_nIntNum].OffsetLow = (unsigned short)((unsigned long)A_pHandler & 0xffff);
		} 
}

/******************************************************************
 * NAME : SetTrapGate
 * SYNOPSIS : initialize trap gate
 * EXTERNAL EFFECTS : IDT[]
 * ARGUMENTS : A_nIntNum trap number
 *             A_pHandler address of trap handler
 * RETURNS : N/A
 * ERRNOS : N/A
 ******************************************************************/
void SetTrapGate(int A_nIntNum, void *A_pHandler)
{
		if (A_nIntNum < IDT_SIZE) {
				IDT[A_nIntNum].OffsetHigh = (unsigned short)((unsigned long)A_pHandler & 0xffff0000) >> 16;
				IDT[A_nIntNum].Type = DESTYPE_TRAP;			/* trap descriptor */
				IDT[A_nIntNum].SegSelector = KERNEL_CODE;	/* kernel code segment */
				IDT[A_nIntNum].OffsetLow = (unsigned short)((unsigned long)A_pHandler & 0xffff);
		} 
}



/******************************************************************
 * NAME : InitIDT
 * SYNOPSIS : initialize INTERRUPT DESCRIPTOR TABLE
 * EXTERNAL EFFECTS : N/A
 * ARGUMENTS :N/A
 * RETURNS : N/A
 * ERRNOS : N/A
 ******************************************************************/
void InitIDT(void)
{
	int nIndex = 0;
	
	/* initialize all IDT with null handler */
	for (nIndex = 0; nIndex < IDT_SIZE; nIndex++) {
		SetTrapGate(nIndex, UnidentifiedINTR);
	}

	/* int 0 : Devide Error Exception */
	SetTrapGate(0, DivideZero);
	/* int 1 : Debug Exception */
	SetTrapGate(1, Debug);
	/* int 2 : NMI interrupt */
	SetIntrGate(2, NMI);
	/* int 3 : Breakpoint exception */
	SetSystemGate(3, BreakPoint);
	/* int 4 : Overflow Exception */
	SetSystemGate(4, Overflow);
	/* int 5 : BOUND Exceeded Exception */
	SetSystemGate(5, BOUNDExceed);
	/* int 6 : Invalid Opcode Exception */
	SetTrapGate(6, InvalidOpcode);
	/* int 7 : Device Not Available Exception */
	SetTrapGate(7, DeviceDisable);
	/* int 8 : Double Fault Exception */
	SetTrapGate(8, DoubleFault);
	/* int 9 : Coprocessor Segment Overrun */
	SetTrapGate(9, CoprocessorSegOver);
	/* int 10 : Invalid TSS Exception */
	SetTrapGate(10, InvalidTSS);
	/* int 11 : Segment Not Present */
	SetTrapGate(11, SegNotPresent);
	/* int 12 : Stack Fault Exception */
	SetTrapGate(12, StackFault);
	/* int 13 : General Protection Exception */
	SetTrapGate(13, GeneralProtection);
	/* int 14 : Page Fault Exception */
	SetIntrGate(14, PageFault);
	/* int 15 : reserved for INTEL */
	SetTrapGate(15, INTELReserved);
	/* int 16 : x87 FPU Floating Point Error */
	SetTrapGate(16, FloatingPoint);
	/* int 17 : Alignment Check Exception */
	SetTrapGate(17, AlignCheck);
	/* int 18 : Machine Check Exception */
	SetTrapGate(18, MachineCheck);
	/* int 19 : SIMD Floationg Point Exception */
	SetTrapGate(19, SIMDFault);

	DECLARE_IRQ(0);	
	DECLARE_IRQ(1);	
	DECLARE_IRQ(2);	
	DECLARE_IRQ(3);	
	DECLARE_IRQ(4);	
	DECLARE_IRQ(5);	
	DECLARE_IRQ(6);	
	DECLARE_IRQ(7);	
	DECLARE_IRQ(8);	
	DECLARE_IRQ(9);	
	DECLARE_IRQ(10);	
	DECLARE_IRQ(11);	
	DECLARE_IRQ(12);	
	DECLARE_IRQ(13);	
	DECLARE_IRQ(14);	
	DECLARE_IRQ(15);	

	//SetSystemGate(SYSCALL_VECTOR, HANDLER_SysCall);
	
}


